@@ -12,6 +12,7 @@
 package deploy
 
 import (
+	"context"
 	"encoding/json"
 	"fmt"
 	"os"
@@ -19,9 +20,18 @@ import (
 
 	orgv1 "github.com/eclipse/che-operator/pkg/apis/org/v1"
 	"github.com/eclipse/che-operator/pkg/util"
+	"github.com/google/go-cmp/cmp"
 	"github.com/sirupsen/logrus"
 	corev1 "k8s.io/api/core/v1"
+	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/types"
+	runtimeClient "sigs.k8s.io/controller-runtime/pkg/client"
+	"sigs.k8s.io/controller-runtime/pkg/controller/controllerutil"
+)
+
+const (
+	CheConfigMapName = "che"
 )
 
 func addMap(a map[string]string, b map[string]string) {
@@ -49,13 +59,13 @@ type CheConfigMap struct {
 	PreCreateSubPaths                      string `json:"CHE_INFRA_KUBERNETES_PVC_PRECREATE__SUBPATHS"`
 	TlsSupport                             string `json:"CHE_INFRA_OPENSHIFT_TLS__ENABLED"`
 	K8STrustCerts                          string `json:"CHE_INFRA_KUBERNETES_TRUST__CERTS"`
-	DatabaseURL                            string `json:"CHE_JDBC_URL"`
-	DbUserName                             string `json:"CHE_JDBC_USERNAME"`
-	DbPassword                             string `json:"CHE_JDBC_PASSWORD"`
+	DatabaseURL                            string `json:"CHE_JDBC_URL,omitempty"`
+	DbUserName                             string `json:"CHE_JDBC_USERNAME,omitempty"`
+	DbPassword                             string `json:"CHE_JDBC_PASSWORD,omitempty"`
 	CheLogLevel                            string `json:"CHE_LOG_LEVEL"`
-	KeycloakURL                            string `json:"CHE_KEYCLOAK_AUTH__SERVER__URL"`
-	KeycloakRealm                          string `json:"CHE_KEYCLOAK_REALM"`
-	KeycloakClientId                       string `json:"CHE_KEYCLOAK_CLIENT__ID"`
+	KeycloakURL                            string `json:"CHE_KEYCLOAK_AUTH__SERVER__URL,omitempty"`
+	KeycloakRealm                          string `json:"CHE_KEYCLOAK_REALM,omitempty"`
+	KeycloakClientId                       string `json:"CHE_KEYCLOAK_CLIENT__ID,omitempty"`
 	OpenShiftIdentityProvider              string `json:"CHE_INFRA_OPENSHIFT_OAUTH__IDENTITY__PROVIDER"`
 	JavaOpts                               string `json:"JAVA_OPTS"`
 	WorkspaceJavaOpts                      string `json:"CHE_WORKSPACE_JAVA__OPTIONS"`
@@ -73,6 +83,51 @@ type CheConfigMap struct {
 	CheJGroupsKubernetesLabels             string `json:"KUBERNETES_LABELS,omitempty"`
 }
 
+type ConfigMapProvisioningStatus struct {
+	ProvisioningStatus
+	ConfigMap *corev1.ConfigMap
+}
+
+func SyncConfigMapToCluster(checluster *orgv1.CheCluster, cheEnv map[string]string, clusterAPI ClusterAPI) ConfigMapProvisioningStatus {
+	specConfigMap, err := GetSpecConfigMap(checluster, cheEnv, clusterAPI)
+	if err != nil {
+		return ConfigMapProvisioningStatus{
+			ProvisioningStatus: ProvisioningStatus{Err: err},
+		}
+	}
+
+	clusterConfigMap, err := getClusterConfigMap(specConfigMap.Name, specConfigMap.Namespace, clusterAPI.Client)
+	if err != nil {
+		return ConfigMapProvisioningStatus{
+			ProvisioningStatus: ProvisioningStatus{Err: err},
+		}
+	}
+
+	if clusterConfigMap == nil {
+		logrus.Infof("Creating a new object: %s, name %s", specConfigMap.Kind, specConfigMap.Name)
+		err := clusterAPI.Client.Create(context.TODO(), specConfigMap)
+		return ConfigMapProvisioningStatus{
+			ProvisioningStatus: ProvisioningStatus{Requeue: true, Err: err},
+		}
+	}
+
+	diff := cmp.Diff(clusterConfigMap.Data, specConfigMap.Data)
+	if len(diff) > 0 {
+		logrus.Infof("Updating existed object: %s, name: %s", specConfigMap.Kind, specConfigMap.Name)
+		fmt.Printf("Difference:\n%s", diff)
+		clusterConfigMap.Data = specConfigMap.Data
+		err := clusterAPI.Client.Update(context.TODO(), clusterConfigMap)
+		return ConfigMapProvisioningStatus{
+			ProvisioningStatus: ProvisioningStatus{Requeue: true, Err: err},
+		}
+	}
+
+	return ConfigMapProvisioningStatus{
+		ProvisioningStatus: ProvisioningStatus{Continue: true},
+		ConfigMap:          clusterConfigMap,
+	}
+}
+
 // GetConfigMapData gets env values from CR spec and returns a map with key:value
 // which is used in CheCluster ConfigMap to configure CheCluster master behavior
 func GetConfigMapData(cr *orgv1.CheCluster) (cheEnv map[string]string) {
@@ -82,8 +137,7 @@ func GetConfigMapData(cr *orgv1.CheCluster) (cheEnv map[string]string) {
 	if err != nil {
 		logrus.Errorf("Failed to get current infra: %s", err)
 	}
-	cheFlavor := util.GetValue(cr.Spec.Server.CheFlavor, DefaultCheFlavor)
-	chePostgresPassword := cr.Spec.Database.ChePostgresPassword
+	cheFlavor := DefaultCheFlavor(cr)
 	infra := "kubernetes"
 	if isOpenShift {
 		infra = "openshift"
@@ -91,10 +145,10 @@ func GetConfigMapData(cr *orgv1.CheCluster) (cheEnv map[string]string) {
 	tls := "false"
 	openShiftIdentityProviderId := "NULL"
 	openshiftOAuth := cr.Spec.Auth.OpenShiftoAuth
-	defaultTargetNamespaceDefault := cr.Namespace  // By default Che SA has right in the namespace where Che in installed ...
+	defaultTargetNamespaceDefault := cr.Namespace // By default Che SA has right in the namespace where Che in installed ...
 	if openshiftOAuth && isOpenShift {
 		// ... But if the workspace is created under the openshift identity of the end-user,
-		// Then we'll have rights to create any new namespace 
+		// Then we'll have rights to create any new namespace
 		defaultTargetNamespaceDefault = "<username>-" + cheFlavor
 		openShiftIdentityProviderId = "openshift-v3"
 		if isOpenshift4 {
@@ -114,6 +168,8 @@ func GetConfigMapData(cr *orgv1.CheCluster) (cheEnv map[string]string) {
 	proxyJavaOpts := ""
 	proxyUser := cr.Spec.Server.ProxyUser
 	proxyPassword := cr.Spec.Server.ProxyPassword
+	proxySecret := cr.Spec.Server.ProxySecret
+
 	nonProxyHosts := cr.Spec.Server.NonProxyHosts
 	if len(nonProxyHosts) < 1 && len(cr.Spec.Server.ProxyURL) > 1 {
 		nonProxyHosts = os.Getenv("KUBERNETES_SERVICE_HOST")
@@ -121,16 +177,25 @@ func GetConfigMapData(cr *orgv1.CheCluster) (cheEnv map[string]string) {
 		nonProxyHosts = nonProxyHosts + "|" + os.Getenv("KUBERNETES_SERVICE_HOST")
 	}
 	if len(cr.Spec.Server.ProxyURL) > 1 {
-		proxyJavaOpts = util.GenerateProxyJavaOpts(cr.Spec.Server.ProxyURL, cr.Spec.Server.ProxyPort, nonProxyHosts, proxyUser, proxyPassword)
+		proxyJavaOpts, err = util.GenerateProxyJavaOpts(cr.Spec.Server.ProxyURL, cr.Spec.Server.ProxyPort, nonProxyHosts, proxyUser, proxyPassword, proxySecret, cr.Namespace)
+		if err != nil {
+			logrus.Errorf("Failed to generate java proxy options: %v", err)
+		}
 	}
 	cheWorkspaceHttpProxy := ""
 	cheWorkspaceNoProxy := ""
 	if len(cr.Spec.Server.ProxyURL) > 1 {
-		cheWorkspaceHttpProxy, cheWorkspaceNoProxy = util.GenerateProxyEnvs(cr.Spec.Server.ProxyURL, cr.Spec.Server.ProxyPort, nonProxyHosts, proxyUser, proxyPassword)
+		cheWorkspaceHttpProxy, cheWorkspaceNoProxy, err = util.GenerateProxyEnvs(cr.Spec.Server.ProxyURL, cr.Spec.Server.ProxyPort, nonProxyHosts, proxyUser, proxyPassword, proxySecret, cr.Namespace)
+		if err != nil {
+			logrus.Errorf("Failed to generate proxy env variables: %v", err)
+		}
 	}
 
 	ingressDomain := cr.Spec.K8s.IngressDomain
 	tlsSecretName := cr.Spec.K8s.TlsSecretName
+	if tlsSupport && tlsSecretName == "" {
+		tlsSecretName = "che-tls"
+	}
 	securityContextFsGroup := util.GetValue(cr.Spec.K8s.SecurityContextFsGroup, DefaultSecurityContextFsGroup)
 	securityContextRunAsUser := util.GetValue(cr.Spec.K8s.SecurityContextRunAsUser, DefaultSecurityContextRunAsUser)
 	pvcStrategy := util.GetValue(cr.Spec.Storage.PvcStrategy, DefaultPvcStrategy)
@@ -144,7 +209,6 @@ func GetConfigMapData(cr *orgv1.CheCluster) (cheEnv map[string]string) {
 		preCreateSubPaths = "false"
 	}
 	chePostgresHostName := util.GetValue(cr.Spec.Database.ChePostgresHostName, DefaultChePostgresHostName)
-	chePostgresUser := util.GetValue(cr.Spec.Database.ChePostgresUser, DefaultChePostgresUser)
 	chePostgresPort := util.GetValue(cr.Spec.Database.ChePostgresPort, DefaultChePostgresPort)
 	chePostgresDb := util.GetValue(cr.Spec.Database.ChePostgresDb, DefaultChePostgresDb)
 	keycloakRealm := util.GetValue(cr.Spec.Auth.IdentityProviderRealm, cheFlavor)
@@ -156,10 +220,11 @@ func GetConfigMapData(cr *orgv1.CheCluster) (cheEnv map[string]string) {
 	cheLogLevel := util.GetValue(cr.Spec.Server.CheLogLevel, DefaultCheLogLevel)
 	cheDebug := util.GetValue(cr.Spec.Server.CheDebug, DefaultCheDebug)
 	cheMetrics := strconv.FormatBool(cr.Spec.Metrics.Enable)
-	cheLabels := util.MapToKeyValuePairs(GetLabels(cr, util.GetValue(cr.Spec.Server.CheFlavor, DefaultCheFlavor)))
+	cheLabels := util.MapToKeyValuePairs(GetLabels(cr, cheFlavor))
+	cheMultiUser := GetCheMultiUser(cr)
 
 	data := &CheConfigMap{
-		CheMultiUser:                           "true",
+		CheMultiUser:                           cheMultiUser,
 		CheHost:                                cheHost,
 		ChePort:                                "8080",
 		CheApi:                                 protocol + "://" + cheHost + "/api",
@@ -177,13 +242,7 @@ func GetConfigMapData(cr *orgv1.CheCluster) (cheEnv map[string]string) {
 		PreCreateSubPaths:                      preCreateSubPaths,
 		TlsSupport:                             tls,
 		K8STrustCerts:                          tls,
-		DatabaseURL:                            "jdbc:postgresql://" + chePostgresHostName + ":" + chePostgresPort + "/" + chePostgresDb,
-		DbUserName:                             chePostgresUser,
-		DbPassword:                             chePostgresPassword,
 		CheLogLevel:                            cheLogLevel,
-		KeycloakURL:                            keycloakURL + "/auth",
-		KeycloakRealm:                          keycloakRealm,
-		KeycloakClientId:                       keycloakClientId,
 		OpenShiftIdentityProvider:              openShiftIdentityProviderId,
 		JavaOpts:                               DefaultJavaOpts + " " + proxyJavaOpts,
 		WorkspaceJavaOpts:                      DefaultWorkspaceJavaOpts + " " + proxyJavaOpts,
@@ -201,6 +260,17 @@ func GetConfigMapData(cr *orgv1.CheCluster) (cheEnv map[string]string) {
 		CheMetricsEnabled:                      cheMetrics,
 	}
 
+	if cheMultiUser == "true" {
+		data.KeycloakURL = keycloakURL + "/auth"
+		data.KeycloakRealm = keycloakRealm
+		data.KeycloakClientId = keycloakClientId
+		data.DatabaseURL = "jdbc:postgresql://" + chePostgresHostName + ":" + chePostgresPort + "/" + chePostgresDb
+		if len(cr.Spec.Database.ChePostgresSecret) < 1 {
+			data.DbUserName = cr.Spec.Database.ChePostgresUser
+			data.DbPassword = cr.Spec.Database.ChePostgresPassword
+		}
+	}
+
 	out, err := json.Marshal(data)
 	if err != nil {
 		fmt.Println(err)
@@ -226,18 +296,43 @@ func GetConfigMapData(cr *orgv1.CheCluster) (cheEnv map[string]string) {
 	return cheEnv
 }
 
-func NewCheConfigMap(cr *orgv1.CheCluster, cheEnv map[string]string) *corev1.ConfigMap {
-	labels := GetLabels(cr, util.GetValue(cr.Spec.Server.CheFlavor, DefaultCheFlavor))
-	return &corev1.ConfigMap{
+func GetSpecConfigMap(checluster *orgv1.CheCluster, cheEnv map[string]string, clusterAPI ClusterAPI) (*corev1.ConfigMap, error) {
+	labels := GetLabels(checluster, DefaultCheFlavor(checluster))
+	configMap := &corev1.ConfigMap{
 		TypeMeta: metav1.TypeMeta{
 			Kind:       "ConfigMap",
 			APIVersion: "v1",
 		},
 		ObjectMeta: metav1.ObjectMeta{
-			Name:      "che",
-			Namespace: cr.Namespace,
+			Name:      CheConfigMapName,
+			Namespace: checluster.Namespace,
 			Labels:    labels,
 		},
 		Data: cheEnv,
 	}
+
+	if !util.IsTestMode() {
+		err := controllerutil.SetControllerReference(checluster, configMap, clusterAPI.Scheme)
+		if err != nil {
+			return nil, err
+		}
+	}
+
+	return configMap, nil
+}
+
+func getClusterConfigMap(name string, namespace string, client runtimeClient.Client) (*corev1.ConfigMap, error) {
+	configMap := &corev1.ConfigMap{}
+	namespacedName := types.NamespacedName{
+		Namespace: namespace,
+		Name:      name,
+	}
+	err := client.Get(context.TODO(), namespacedName, configMap)
+	if err != nil {
+		if errors.IsNotFound(err) {
+			return nil, nil
+		}
+		return nil, err
+	}
+	return configMap, nil
 }
