@@ -13,37 +13,62 @@ package deploy
 
 import (
 	"fmt"
+	"io/ioutil"
 	"os"
 	"testing"
 
 	orgv1 "github.com/eclipse/che-operator/pkg/apis/org/v1"
+	"gopkg.in/yaml.v2"
+	appsv1 "k8s.io/api/apps/v1"
 )
 
-
-const (
-	cheVersionTest           = "2.1"
-	cheServerImageTest       = "registry.redhat.io/codeready-workspaces/server-rhel8:2.1"
-	pluginRegistryImageTest  = "registry.redhat.io/codeready-workspaces/pluginregistry-rhel8:2.1"
-	devfileRegistryImageTest = "registry.redhat.io/codeready-workspaces/devfileregistry-rhel8:2.1"
-	pvcJobsImageTest         = "registry.access.redhat.com/ubi8-minimal:8.1-398"
-	postgresImageTest        = "registry.redhat.io/rhscl/postgresql-96-rhel7:1-47"
-	keycloakImageTest        = "registry.redhat.io/redhat-sso-7/sso73-openshift:1.0-15"
-	brokerMetadataTest		 = "registry.redhat.io/codeready-workspaces/pluginbroker-metadata-rhel8:2.1"
-	brokerArtifactsTest		 = "registry.redhat.io/codeready-workspaces/pluginbroker-artifacts-rhel8:2.1"
-	jwtProxyTest		 	 = "registry.redhat.io/codeready-workspaces/jwtproxy-rhel8:2.1"
+var (
+	cheVersionTest           string
+	cheServerImageTest       string
+	pluginRegistryImageTest  string
+	devfileRegistryImageTest string
+	pvcJobsImageTest         string
+	postgresImageTest        string
+	keycloakImageTest        string
+	brokerMetadataTest       string
+	brokerArtifactsTest      string
+	jwtProxyTest             string
+	tlsJobImageTest          string
 )
 
 func init() {
-	os.Setenv("CHE_VERSION", cheVersionTest)
-	os.Setenv("IMAGE_default_che_server", cheServerImageTest)
-	os.Setenv("IMAGE_default_plugin_registry", pluginRegistryImageTest)
-	os.Setenv("IMAGE_default_devfile_registry", devfileRegistryImageTest)
-	os.Setenv("IMAGE_default_pvc_jobs", pvcJobsImageTest)
-	os.Setenv("IMAGE_default_postgres", postgresImageTest)
-	os.Setenv("IMAGE_default_keycloak", keycloakImageTest)
-	os.Setenv("IMAGE_default_che_workspace_plugin_broker_metadata", brokerMetadataTest)
-	os.Setenv("IMAGE_default_che_workspace_plugin_broker_artifacts", brokerArtifactsTest)
-	os.Setenv("IMAGE_default_che_server_secure_exposer_jwt_proxy_image", jwtProxyTest)
+	operator := &appsv1.Deployment{}
+	data, err := ioutil.ReadFile("../../deploy/operator.yaml")
+	yaml.Unmarshal(data, operator)
+	if err == nil {
+		for _, env := range operator.Spec.Template.Spec.Containers[0].Env {
+			os.Setenv(env.Name, env.Value)
+			switch env.Name {
+			case "CHE_VERSION":
+				cheVersionTest = env.Value
+			case "IMAGE_default_che_server":
+				cheServerImageTest = env.Value
+			case "IMAGE_default_plugin_registry":
+				pluginRegistryImageTest = env.Value
+			case "IMAGE_default_devfile_registry":
+				devfileRegistryImageTest = env.Value
+			case "IMAGE_default_che_tls_secrets_creation_job":
+				tlsJobImageTest = env.Value
+			case "IMAGE_default_pvc_jobs":
+				pvcJobsImageTest = env.Value
+			case "IMAGE_default_postgres":
+				postgresImageTest = env.Value
+			case "IMAGE_default_keycloak":
+				keycloakImageTest = env.Value
+			case "IMAGE_default_che_workspace_plugin_broker_metadata":
+				brokerMetadataTest = env.Value
+			case "IMAGE_default_che_workspace_plugin_broker_artifacts":
+				brokerArtifactsTest = env.Value
+			case "IMAGE_default_che_server_secure_exposer_jwt_proxy_image":
+				jwtProxyTest = env.Value
+			}
+		}
+	}
 
 	InitDefaultsFromEnv()
 }
@@ -98,9 +123,9 @@ func TestDefaultFromEnv(t *testing.T) {
 
 func TestCorrectImageName(t *testing.T) {
 	testCases := map[string]string{
-		"registry.redhat.io/codeready-workspaces/server-operator-rhel8:1.2": "server-operator-rhel8:1.2",
-		"registry.redhat.io/codeready-workspaces/server-operator-rhel8:2.0": "server-operator-rhel8:2.0",
-		"registry.redhat.io/codeready-workspaces/crw-2-rhel8-operator:2.1":  "crw-2-rhel8-operator:2.1",
+		"registry.redhat.io/codeready-workspaces/crw-2-rhel8-operator:latest":  "crw-2-rhel8-operator:latest",
+		"eclipse/che-operator:7.1.0":            "che-operator:7.1.0",
+		"registry.redhat.io/codeready-workspaces/crw-2-rhel8-operator:2.2":  "crw-2-rhel8-operator:2.2",
 	}
 	for k, v := range testCases {
 		t.Run(k, func(*testing.T) {
